# Info

Moodle connector for midPoint (ConnId framework).

## Connector Objects Schema

### User

Operational attributes:

* **user_id**: (\_\_UID\_\_ - read-only, \_\_NAME\_\_): user id (*id* column in *mdl_user* table)

Attributes:

* **auth** (required): authentization type ("manual" is the Moodle default)
* **mnethostid**: username tenant
* **username**
* **idnumber**
* **firstname**
* **lastname**
* **email**
* **institution**
* **confirmed**
* **deleted**
* **password**
* **lang** (required): language ("en" is the Moodle default)
* **personal_number**: from user attributes

Although only **mnethostid**+**username** combination is requied to be unique, connector checks uniqueness of **username** attribute globally.

### Cohort

Operational attributes:

* **cohort_id** (**\_\_UID\_\_**, read-only): the primary key, *id* column in *mdl_cohort* table
* **name** (**\_\_NAME\_\_**): unique group name

Attributes:

* **contextid**
* **idnumber**
* **description**
* **descriptionformat**
* **visible**
* **component**
* **theme**
* **timecreated**: creation time (read-only, set by connector)
* **timemodified**: creation or modification time (read-only, set by connector)

## Operations

Operations are performed on *mdl_user* and *mdl_user_attribute* tables.

### Create

Create record in *mdl_user* and optionally in *mdl_user_attribute* table. Doesn't fail if record in *mdl_user_attribute* table already exists.

### Delete

Behaviour depends on the *deleteMethod* configuration value.

1. *purge* (default): records are really deleted
2. *recycle*: record is kept, only **username** and **email** is modified with added timestamp to support user name recyclation and attributes are set to *NULL*, check also the *userSuffix* configuration parameter

### Search

Supported filters:

* *contains* + not
* *equals* + not
* *starsWith* + not
* *endsWith* + not
* *and*

### Test

### Update Delta

Renaming is supported by **\_\_NAME\_\_** attribute.

Empty strings are allowed. Most of the optional attributes are nulled by setting them to "", except attributes in *mdl_user_attribute* table (like **personal_number**).

There is the config option *forceCleanup*, which enables a cleanup of the users with conflicting **username** using configured deleted method (=*purge* or *recycle*). Only users with **user_id** >= 1000000 are cleaned.

## Build

Requirements:

* [connector-parent (cz.zcu.connectors)](https://ipmil.civ.zcu.cz/midpoint/connector-parent)
* [connector-utils (cz.zcu.connectors)](https://ipmil.civ.zcu.cz/midpoint/connector-utils)

Steps:

    mvn package

## Tests

1. update test parameters in *src/test/resources/cz.zcu.connectors.moodle.MoodleConnector/config/config.groovy*

2. launch tests:

    mvn package -P mariadb
