package cz.zcu.connectors.moodle;

import org.identityconnectors.common.Assertions;
import org.identityconnectors.framework.spi.ConfigurationProperty;

import cz.zcu.connectors.DbConfiguration;


public class MoodleConfiguration extends DbConfiguration {
	// keep in sync with src/main/resources/Messages.properties
	/** Default value for database table prefix. */
	public static final String DEFAULT_TABLE_PREFIX = "";
	public static final String DELETE_METHOD_PURGE = "purge";
	public static final String DELETE_METHOD_RECYCLE = "recycle";

	/** Database table prefix (without the dot). */
	private String tablePrefix = DEFAULT_TABLE_PREFIX;
	private String deleteMethod = DELETE_METHOD_PURGE;
	private String userSuffix = "";
	private Boolean forcedCleanup = false;


	/**
	 * Get database table prefix (the namespace, without dot).
	 *
	 * @return the table prefix
	 */
	@ConfigurationProperty(order = DbConfiguration.PROPERTIES_COUNT + 1, displayMessageKey = "moodle.tablePrefix.display",
			groupMessageKey = "basic.group", helpMessageKey = "moodle.tablePrefix.help")
	public String getTablePrefix() {
		return tablePrefix;
	}


	/**
	 * Set database table prefix (the namespace, without dot).
	 *
	 * @param tablePrefix the table prefix
	 */
	public void setTablePrefix(final String tablePrefix) {
		this.tablePrefix = tablePrefix;
	}


	@ConfigurationProperty(order = DbConfiguration.PROPERTIES_COUNT + 2, displayMessageKey = "moodle.deleteMethod.display",
			groupMessageKey = "basic.group", helpMessageKey = "moodle.deleteMethod.help")
	public String getDeleteMethod() {
		return deleteMethod;
	}


	public void setDeleteMethod(String value) {
		this.deleteMethod = value;
	}


	@ConfigurationProperty(order = DbConfiguration.PROPERTIES_COUNT + 3, displayMessageKey = "moodle.userSuffix.display",
		groupMessageKey = "basic.group", helpMessageKey = "moodle.userSuffix.help")
	public String getUserSuffix() {
		return userSuffix;
	}


	public void setUserSuffix(String userSuffix) {
		this.userSuffix = userSuffix;
	}


	@ConfigurationProperty(order = DbConfiguration.PROPERTIES_COUNT + 4, displayMessageKey = "moodle.forcedCleanup.display",
		groupMessageKey = "basic.group", helpMessageKey = "moodle.forcedCleanup.help")
	public Boolean getForcedCleanup() {
		return forcedCleanup;
	}


	public void setForcedCleanup(Boolean forcedCleanup) {
		this.forcedCleanup = forcedCleanup;
	}


	/**
	 * {@inheritDoc}
	 */
	public void validate() {
		Assertions.nullCheck(tablePrefix, "tablePrefix");
		Assertions.nullCheck(deleteMethod, "deleteMethod");
	}
}
