package cz.zcu.connectors.moodle;

import java.time.ZoneId;
import java.time.ZoneOffset;

import cz.zcu.connectors.Db;


/**
 * Connector Moodle connection.
 */
public class MoodleConnection {
	/** Database connection. */
	public Db db;
	/** The time zone to use (system default). */
	public ZoneId zone = ZoneOffset.systemDefault();
	public String sqlNow;


	/**
	 * Create a connection.
	 *
	 * @param config connector configuration
	 */
	public MoodleConnection(final MoodleConfiguration config) {
		db = new Db("moodle", config);
	}


	/**
	 * Close and free database connection.
	 */
	public void close() {
		if (db != null) {
			db.close();
		}
		db = null;
	}


	/**
	 * Quick check alive on the connection.
	 */
	public void checkAlive() {
		db.checkAlive();
	}


	/**
	 * Get database utils handler class.
	 *
	 * @return DB handler class
	 */
	public Db getDb() {
		return this.db;
	}


	/**
	 * Get SQL snippet for current timestamp.
	 *
	 * @return SQL code snippet
	 */
	public String getSqlNow() {
		return "UNIX_TIMESTAMP()";
	}


	/**
	 * The connection time zone (system default).
	 *
	 * @return connection zone id
	 */
	public ZoneId getZone() {
		return this.zone;
	}

}
