package cz.zcu.connectors.moodle;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.identityconnectors.common.logging.Log;
import org.identityconnectors.framework.common.exceptions.AlreadyExistsException;

import cz.zcu.connectors.Db;


/**
 * Utils for Moodle connector.
 */
public class Utils {
	/** Logging. */
	private static final Log l = Log.getLog(Utils.class);


	protected Utils() {
		throw new UnsupportedOperationException();
	}


	/**
	 * Check the table, if the value in column exists.
	 *
	 * @param msg Error message
	 * @param db Connector utils context
	 * @param table table name
	 * @param uid UID of the user, where the value is permitted (for update)
	 * @param column field name
	 * @param value tested value
	 * @throws SQLException any SQL error
	 * @throws AlreadyExistsException value already exists in the table
	 */
	public static void checkDuplicity(final String msg, final Db db, final String table, final String uid, final String idColumn, final String column, final String value) throws SQLException {
		PreparedStatement st;
		ResultSet rs;
		final String sql = "SELECT " + idColumn + ", " + column + " FROM " + table + " WHERE " + column + " = ?";

		st = db.create(sql);
		st.setString(1, value);
		l.ok("SQL: {0} [{1}]", sql, value);
		rs = st.executeQuery();
		while (rs.next()) {
			String uid2 = Long.toString(rs.getLong(1));
			if (uid == null || !uid.equals(uid2)) {
				rs.close();
				throw new AlreadyExistsException(msg + " with " + column + "=" + value + " already exists");
			}
		}
		rs.close();
	}


	/**
	 * Fetch generated ID from the executed JDBC statement.
	 *
	 * @param db DB context class
	 * @param st the statement
	 * @param idField ID column name
	 * @param table table name
	 * @return generated ID or null
	 * @throws SQLException SQL exception
	 */
	public static String lastId(final Db db, final PreparedStatement st, final String idField, final String table) throws SQLException {
		String id = null;
		ResultSet rs = st.getGeneratedKeys();
		if (rs.next()) {
			id = rs.getString(1);
			// workaround for Oracle: it is actually the ROWID
			if (db.getDbType() == Db.ORACLE) {
				l.ok("SQL: ROWID {0}", id);
				PreparedStatement stId = db.create("SELECT " + idField + " FROM " + table + " WHERE rowid = ?");
				stId.setString(1, id);
				ResultSet rsId = stId.executeQuery();
				if (rsId.next()) {
					id = rsId.getString(1);
				} else {
					id = null;
				}
				rsId.close();
			}
		}
		rs.close();
		l.ok("SQL: generated ID {0}", id);
		return id;
	}

}
