package cz.zcu.connectors.moodle;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

import org.identityconnectors.common.logging.Log;
import org.identityconnectors.framework.api.ConnectorFacade;
import org.identityconnectors.framework.common.exceptions.AlreadyExistsException;
import org.identityconnectors.framework.common.exceptions.UnknownUidException;
import org.identityconnectors.framework.common.objects.AttributeBuilder;
import org.identityconnectors.framework.common.objects.AttributeInfo;
import org.identityconnectors.framework.common.objects.Name;
import org.identityconnectors.framework.common.objects.ObjectClass;
import org.identityconnectors.framework.common.objects.ObjectClassInfo;
import org.identityconnectors.framework.common.objects.Schema;
import org.identityconnectors.framework.common.objects.Uid;
import org.identityconnectors.framework.common.objects.filter.Filter;
import org.identityconnectors.framework.common.objects.filter.FilterBuilder;
import org.testng.Assert;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import cz.zcu.connectors.moodle.dao.Cohort;


public class MoodleConnectorCohortTests extends MoodleConnectorTestBase {
	/** Logging. */
	private static final Log l = Log.getLog(MoodleConnectorCohortTests.class);
	/** Test object. */
	private Cohort cohort;
	/** Test object for write. */
	private Cohort cohortWrite;
	/** Test object for write. */
	private Cohort cohortWrite2;
	/** Unknown non-existing test object. */
	private Cohort cohortUnknown;


	/**
	 * Test class constructor.
	 */
	public MoodleConnectorCohortTests() {
		super(Cohort.OBJECT_CLASS);
	}


	/**
	 * Tests setup.
	 */
	@BeforeClass
	public void setUp() {
		super.setUp();

		cohort = loadCohort("cohort");
		cohortUnknown = loadCohort("cohortUnknown");
		cohortWrite = loadCohort("cohortWrite");
		cohortWrite2 = loadCohort("cohortWrite2");

		delete(getFacade(null), cohortWrite);
		delete(getFacade(null), cohortWrite2);
	}


	/**
	 * Load Cohort object from testsuite properties.
	 *
	 * @param targetCohort data object to fill
	 * @param name test object name
	 */
	private void loadCohort(final Cohort targetCohort, final String name) {
		load(targetCohort, name, Cohort.ATTRS);
		//if (targetCohort.data.containsKey(Cohort.ATTR_AKTIVNI))
		//	targetCohort.setEnable(targetCohort.get(Cohort.ATTR_AKTIVNI));
		//targetCohort.remove(Cohort.ATTR_AKTIVNI);
		targetCohort.setUid(targetCohort.get(Cohort.ATTR_ID));
		targetCohort.setName(targetCohort.get(Cohort.ATTR_NAME));
	}


	/**
	 * Load Cohort object from testsuite properties.
	 *
	 * @param name test object name
	 * @return loaded Cohort object
	 */
	private Cohort loadCohort(final String name) {
		final Cohort targetCohort = new Cohort();
		loadCohort(targetCohort, name);
		return targetCohort;
	}


	/**
	 * Validate Cohort object.
	 */
	@Test
	public void validateCohortTest() {
		cohort.validate();
		cohortWrite.validate();
	}


	/**
	 * Validate unknown Cohort object.
	 */
	@Test
	public void validateUnknownCohortTest() {
		cohortUnknown.validate();
	}



	/**
	* Get Cohort object test.
	*/
	@Test
	public void getCohortTest() {
		super.getTest(cohort);
	}


	/**
	* Get unknown Cohort object test.
	*/
	@Test
	public void getUnknownCohortTest() {
		super.getUnknownTest(cohortUnknown);
	}


	/**
	 * Search exact Cohort test.
	 */
	@Test
	public void searchExactCohortTest() {
		Filter[] filters = {
			FilterBuilder.equalTo(new Uid(cohort.getUid())),
			FilterBuilder.equalTo(new Name(cohort.getName())),
			FilterBuilder.equalTo(AttributeBuilder.build(Cohort.ATTR_ID, (String) cohort.get(Cohort.ATTR_ID))),
			FilterBuilder.equalTo(AttributeBuilder.build(Cohort.ATTR_NAME, (String) cohort.get(Cohort.ATTR_NAME))),
		};
		super.searchExactTest(cohort, filters);
	}


	/**
	 * Search all Cohort test.
	 */
	@Test
	public void searchAllCohortTest() {
		super.searchAllTest(cohort);
	}


	/**
	 * Create Cohort test.
	 */
	@Test
	public void createCohortTest() {
		super.createTest(cohortWrite);
	}


	/**
	 * Delete Cohort test.
	 */
	@Test
	public void deleteCohortTest() {
		super.deleteTest(cohortWrite);
	}


	/**
	 * Delete unknown Cohort test.
	 */
	@Test(expectedExceptions = { UnknownUidException.class })
	public void deleteUnknownCohortTest() {
		super.deleteUnknownTest(cohortUnknown);
	}


	/**
	 * Update Cohort test.
	 */
	@Test
	public void updateCohortTest() {
		super.updateTest(cohortWrite, cohortWrite2, true, false);
	}


	/**
	 * Update unknown Cohort test.
	 */
	@Test(expectedExceptions = { UnknownUidException.class })
	public void updateUnknownCohortTest() {
		super.updateTest(cohortUnknown, cohortWrite2, false, false);
	}


	/**
	 * Update with renaming Cohort test.
	 */
	@Test
	public void updateWithRenameCohortTest() {
		super.updateTest(cohortWrite, cohortWrite2, true, true);
	}


	/**
	 * Rename UID Cohort test.
	 */
	@Test
	public void renameUidCohortTest() {
		super.renameUidTest(cohortWrite, cohortWrite2);
	}


	/**
	 * Rename Cohort test.
	 */
	@Test
	public void renameCohortTest() {
		super.renameTest(cohortWrite, cohortWrite2);
	}


	/**
	 * Rename exists Cohort test.
	 */
	@Test(expectedExceptions = { AlreadyExistsException.class })
	public void renameExistsCohortTest() {
		super.renameExistsTest(cohortWrite, cohortWrite2);
	}


	/**
	 * Update reset Cohort test.
	 */
	@Test
	public void updateResetCohortTest() {
		final String[] nullableAttrs = {
				Cohort.ATTR_DESCRIPTION,
				Cohort.ATTR_THEME,
		};
		super.updateResetTest(cohortWrite, nullableAttrs);
	}


	/**
	 * Get latest sync Cohort test.
	 */
	@Test
	public void getLatestSyncCohortTest() {
		super.getLatestSyncTest(Cohort.OBJECT_CLASS, Cohort.OBJECT_CLASS, cohortWrite, cohortWrite2, new Cohort(), Cohort.ATTR_CONTEXTID);
	}


	/**
	 * Synchronization Cohort test.
	 */
	@Test
	public void syncCohortTest() {
		super.syncTest(Cohort.OBJECT_CLASS, Cohort.OBJECT_CLASS, cohortWrite, cohortWrite2, new Cohort(), Cohort.ATTR_CONTEXTID);
	}


	/**
	 * Synchronization Cohort test using ObjectClass.ALL.
	 */
	@Test
	public void syncAllTest() {
		super.syncTest(ObjectClass.ALL, Cohort.OBJECT_CLASS, cohortWrite, cohortWrite2, new Cohort(), Cohort.ATTR_CONTEXTID);
	}


	/**
	 * Cohort schema test.
	 */
	@Test
	public void schemaTest() {
		final ConnectorFacade facade = getFacade(null);
		ObjectClassInfo info;
		Set<String> expected;
		final String[] expectedArray = {
			Name.NAME,
			Uid.NAME,
			// operational: Cohort.ATTR_NAME,
			Cohort.ATTR_CONTEXTID,
			Cohort.ATTR_IDNUMBER,
			Cohort.ATTR_DESCRIPTION,
			Cohort.ATTR_DESCRIPTIONFORMAT,
			Cohort.ATTR_VISIBLE,
			Cohort.ATTR_COMPONENT,
			Cohort.ATTR_THEME,
			Cohort.ATTR_TIMECREATED,
		};

		final Schema schema = facade.schema();

		expected = new HashSet<>(Arrays.asList(expectedArray));
		info = schema.findObjectClassInfo(Cohort.OBJECT_CLASS_NAME);
		for (final AttributeInfo a : info.getAttributeInfo()) {
			l.info("attribute {0}", a.getName());
			switch (a.getName()) {
			case "__NAME__":
				Assert.assertEquals(Cohort.ATTR_NAME, a.getNativeName(), "__NAME__ is " + Cohort.ATTR_NAME);
				break;
			case "__UID__":
				Assert.assertEquals(Cohort.ATTR_ID, a.getNativeName(), "__UID__ is " + Cohort.ATTR_ID);
				break;
			default:
			}
			expected.remove(a.getName());
			// forbidden in midPoint
			Assert.assertNotEquals(a.getName(), "id");
		}
		l.info("remaining missed attributes: {0}", expected);
		Assert.assertTrue(expected.isEmpty(), "no missing attributes from cohort");
	}

}
